package com.docutainsdk

/* nur zum Testen  ReadableMapDump */
import android.app.Activity
import android.content.Intent
import android.net.Uri
import android.util.Log
import com.facebook.react.bridge.ActivityEventListener
import com.facebook.react.bridge.BaseActivityEventListener
import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.bridge.ReadableMapKeySetIterator
import com.facebook.react.bridge.ReadableType
import de.docutain.sdk.Document
import de.docutain.sdk.DocutainSDK
import de.docutain.sdk.Logger
import de.docutain.sdk.dataextraction.AnalyzeConfiguration
import de.docutain.sdk.dataextraction.DocumentDataReader
import de.docutain.sdk.ui.DocumentScanner
import de.docutain.sdk.ui.DocumentScannerConfiguration
import java.io.File

class DocutainSdkModule(reactContext: ReactApplicationContext) :
  ReactContextBaseJavaModule(reactContext) {

  private val logTag = "DocutainSdk"
  private val logTagTest = "DocutainSdkFull"
  private val SCAN_DOCUMENT_RESULT_CODE = 1
  private var callerContext: ReactApplicationContext;
  private var scanPromise : Promise? = null


  private val mActivityEventListener: ActivityEventListener = object : BaseActivityEventListener() {
    override fun onActivityResult(activity: Activity?, requestCode: Int, resultCode: Int, intent: Intent?) {
      Log.v(logTagTest, "onActivityResult" )
      if (requestCode == SCAN_DOCUMENT_RESULT_CODE) {
        if (scanPromise != null) {
          scanPromise!!.resolve(if(resultCode == Activity.RESULT_OK) true else false)
          scanPromise = null
          Log.v(logTagTest, if(resultCode == Activity.RESULT_OK) "scannen finished" else "scannen canceled" )
        }
      }
    }
  }

  init   {
    callerContext = reactContext
    callerContext.addActivityEventListener(mActivityEventListener)
  }

  override fun getName(): String {
    return NAME
  }

  @ReactMethod
  fun initSDK(licenseKey: String, promise: Promise) {
    Log.v(logTagTest, "InitSDK Start")

    if(!DocutainSDK.initSDK(callerContext.getApplicationContext() as android.app.Application, licenseKey)){
      //init of Docutain SDK failed, get the last error message
      Log.e(logTag,"Initialization of the Docutain SDK failed: ${DocutainSDK.getLastError()}")
      //your logic to deactivate access to SDK functionality
      if(licenseKey == "YOUR_LICENSE_KEY_HERE"){
        licenseKeyMissing = true
      }
      promise.resolve(false);
      return;
    }
    else
    {
      Log.d(logTagTest, "initSDK Ok: $licenseKey")
    }

    /*prüfen ob bereits in INitSDK enthalten, falls niht dort implementieren */
    Logger.setLogLevel(Logger.Level.VERBOSE)

    Log.v(logTagTest, "initSDK return true")

    promise.resolve(true);
  }
  @ReactMethod
  fun deleteTempFiles(deleteTraceFileContent: Boolean ,promise: Promise) {
    Log.v(logTagTest, "deleteTempFiles")
    try {
      Log.v(logTagTest, "deleteTraceFileContent")
      val rc = DocutainSDK.deleteTempFiles(deleteTraceFileContent)
      promise.resolve(rc)
    }
    catch(ex:Exception )
    {
      Log.e(logTagTest, "deleteTempFiles Exception"+ ex)
    }
  }
  @ReactMethod
  fun getLastError( promise: Promise) {
    Log.v(logTagTest, "getLastError")
    try {
      val text = DocutainSDK.getLastError()
      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.d(logTagTest, "getLastError Exception"+ ex)
    }
  }
  @ReactMethod
  fun setLogLevel(LogLevel: Int,  promise: Promise) {
    Log.v(logTagTest, "setLogLevel")
    try {
      val logLevel: Logger.Level = Logger.Level.values()[LogLevel];
      Log.d(logTagTest, "setLogLevel " + logLevel)
      Logger.setLogLevel(logLevel);
      promise.resolve(true)
    }
    catch(ex:Exception )
    {
      Log.d(logTagTest, "setLogLevel Exception"+ ex)
    }
  }
  @ReactMethod
  fun getTraceFile( promise: Promise) {
    Log.v(logTagTest, "getTraceFile")
    try {
      val text = Logger.getTraceFile().getPath();
      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.d(logTagTest, "getLastError Exception"+ ex)
    }
  }

  @ReactMethod
  fun setAnalyzeConfiguration(config: ReadableMap, promise: Promise) {
    Log.v(logTagTest, "setAnalyzeConfiguration")
    val analyzeConfig = AnalyzeConfiguration()
    mapAnalyzeConfiguration(config, analyzeConfig);
    Log.v(logTagTest, "analyzeConfig.readBIC:" + analyzeConfig.readBIC)
    if(!DocumentDataReader.setAnalyzeConfiguration(analyzeConfig)){
      Log.e(logTag,"Setting AnalyzeConfiguration failed: ${DocutainSDK.getLastError()}")
      promise.resolve(false);
    }
    else {
      promise.resolve(true);
    }
  }

  @ReactMethod
  fun scanDocument(options: ReadableMap, promise: Promise) {
    Log.v(logTagTest, "scan Start")
    try {
      val currentActivity: Activity? = getCurrentActivity()
      if (currentActivity == null) {
        promise.resolve(false)
        return;
      }
      scanPromise = promise
      val nativeScanConfig = DocumentScannerConfiguration()
      mapDocumentScannerConfiguration(options, nativeScanConfig);
      Log.d(logTagTest, "allowCaptureModeSetting:" + nativeScanConfig.allowCaptureModeSetting)
      Log.d(logTagTest, "theme:" + nativeScanConfig.theme)
      val intent = DocumentScanner.newScanIntent(callerContext.getApplicationContext(), nativeScanConfig)
      currentActivity.startActivityForResult(intent, SCAN_DOCUMENT_RESULT_CODE)
    }
    catch(ex:Exception )
    {
      Log.e(logTagTest, "scannen Exception"+ ex)
    }
    Log.v(logTagTest, "scannen start OK")
  }

  @ReactMethod
  fun loadFile(filepath: String, promise: Promise) {
    Log.v(logTagTest, "loadFile:" + filepath)
    try {
      val fileUri:Uri = Uri.parse(filepath)
      val rc = DocumentDataReader.loadFile(fileUri)
      Log.d(logTagTest, "loadFile rc:" + rc)
      promise.resolve(rc)
    }
    catch(ex:Exception )
    {
      Log.e(logTagTest, "loadFile Exception"+ ex)
    }
  }

  @ReactMethod
  fun writePDF(filepath: String, overWrite: Boolean, PageFormat: Int, promise: Promise) {
    Log.v(logTagTest, "writePDF x:" + filepath)
    Log.d(logTagTest, "PageFormat:" + PageFormat)
    try {
      val PDFFormat: Document.PDFPageFormat = Document.PDFPageFormat.values()[PageFormat];
      Log.d(logTagTest, "PDFFormat:" + PDFFormat)

//      val fileUri:Uri = Uri.parse(filepath)
//      val file2 = File(callerContext.getApplicationContext().filesDir, "SamplePDF")
//      Log.d(logTagTest, "writePDF path:" + file2.getPath())

      val file = File(filepath);
      val rc = Document.writePDF(file,overWrite, de.docutain.sdk.Document.PDFPageFormat.A4)
      Log.d(logTagTest, "writePDF rc:" + rc)
      Log.d(logTagTest, "writePDF rc absolutePath:" + rc?.absolutePath)
      Log.d(logTagTest, "writePDF rc.getPath:" + rc?.getPath())
      promise.resolve(rc?.absolutePath)
    }
    catch(ex:Exception )
    {
      Log.e(logTagTest, "writePDF Exception"+ ex)
    }
  }

  @ReactMethod
  fun getText( promise: Promise) {
    Log.d(logTagTest, "getText")
    try {
      val text = DocumentDataReader.getText()
      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.d(logTagTest, "getText Exception"+ ex)
    }
  }

  @ReactMethod
  fun analyze( promise: Promise) {
    Log.d(logTagTest, "analyze")
    try {
      val text = DocumentDataReader.analyze()
      Log.d(logTagTest, "analyze:" + text)

      promise.resolve(text)
    }
    catch(ex:Exception )
    {
      Log.d(logTagTest, "analyze Exception"+ ex)
    }
  }

  companion object {
    const val NAME = "DocutainSdk"
    var licenseKeyMissing = false
  }

  private fun mapAnalyzeConfiguration(config: ReadableMap, analyzeConfig: AnalyzeConfiguration) {
    dumpReadableMap(config)

    if(config.hasKey("readBIC"))
      analyzeConfig.readBIC = config.getBoolean("readBIC")
    if(config.hasKey("readPaymentState"))
      analyzeConfig.readPaymentState = config.getBoolean("readPaymentState")
  }
  private fun mapDocumentScannerConfiguration(options: ReadableMap, scanConfig: DocumentScannerConfiguration) {
    dumpReadableMap(options)
    //    public final var onboardingImageSource: kotlin.Int /* compiled code */

    //  public final var pageEditConfig: de.docutain.sdk.ui.PageEditConfiguration /* compiled code */
    if(options.hasKey("pageEditConfig")) {
      val optionsPallowCaptureModeSetting = options.getMap("pageEditConfig")
      if(optionsPallowCaptureModeSetting != null)
      {
        if(optionsPallowCaptureModeSetting.hasKey("allowPageFilter")) {
          scanConfig.pageEditConfig.allowPageFilter =
            optionsPallowCaptureModeSetting.getBoolean("allowPageFilter")
        }
        if(optionsPallowCaptureModeSetting.hasKey("allowPageRotation")) {
          scanConfig.pageEditConfig.allowPageRotation = optionsPallowCaptureModeSetting.getBoolean("allowPageRotation")
        }
      }
    }

    if(options.hasKey("allowCaptureModeSetting"))
      scanConfig.allowCaptureModeSetting = options.getBoolean("allowCaptureModeSetting")
    if(options.hasKey("theme"))
      scanConfig.theme = options.getInt("theme")
  }

  private fun dumpReadableMap(readableMap: ReadableMap ){
    var iterator: ReadableMapKeySetIterator
    iterator = readableMap.keySetIterator()
    while (iterator.hasNextKey()) {
      val key:String = iterator.nextKey()
      val type:ReadableType = readableMap.getType (key)
      Log.d(logTagTest, "dumpReadableMap " + key + ": " +  type)
    }
  }

}
