import DocutainSdk

@objc(DocutainSdk)
class DocutainSdk: NSObject, ScanDelegate {

private var scanResolve : RCTPromiseResolveBlock? = nil

  @objc(multiply:withB:withResolver:withRejecter:)
  func multiply(a: Float, b: Float, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void {
    resolve(a*b)
  }

  @objc(initSDK:withResolver:withRejecter:)
  func initSDK(licenseKey: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    if(!DocutainSDK.initSDK(licenseKey: licenseKey)){
      resolve(false)
    } else{
      resolve(true)
    }
  }

  @objc(getLastError:withRejecter:)
  func getLastError(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    resolve(DocutainSDK.getLastError())
  }

  @objc(setAnalyzeConfiguration:withResolver:withRejecter:)
  func setAnalyzeConfiguration(config: [String: Any], resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    //only available for iOS >= 13
    if #available(iOS 13, *){
      let analyzeConfig = AnalyzeConfiguration()
      mapAnalyzeConfiguration(config: config, analyzeConfig: analyzeConfig)
      if(!DocumentDataReader.setAnalyzeConfiguration(analyzeConfiguration: analyzeConfig)){
        resolve(false)
      } else{
        resolve(true)
      }
    } else{
      resolve(false)
    }
  }

  @objc(scanDocument:withResolver:withRejecter:)
  func scanDocument(config: [String: Any], resolve:@escaping RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
    let scanConfig = DocumentScannerConfiguration()
    mapDocumentScannerConfiguration(config: config, scanConfig: scanConfig)
    scanResolve = resolve
    //react runs async, so we need to dispatch on ui thread
    DispatchQueue.main.async {
        UI.scanDocument(scanDelegate: self, scanConfig: scanConfig)
    }
  }

  @objc(loadFile:withResolver:withRejecter:)
  func loadFile(path: String, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      //only available for iOS >= 13
      if #available(iOS 13, *){
          if let url = URL(string: path){
              if(!DocumentDataReader.loadFile(fileUrl: url as URL)){
                resolve(false)
              } else{
                resolve(true)
              }
          } else{
              resolve(false)
          }
      } else{
          resolve(false)
      }
  }

  @objc(writePDF:withOverwrite:withPageFormat:withResolver:withRejecter:)
  func writePDF(filepath: String, overWrite: Bool, pageFormat: Int, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      if let pdfFormat = Document.PDFPageFormat(rawValue: pageFormat){
          if let url = URL(string: filepath){
              if let returnURL = Document.writePDF(fileUrl: url.deletingLastPathComponent(), fileName: url.lastPathComponent, overwrite: overWrite, pageFormat: pdfFormat){
              resolve(returnURL.path)
            } else{
              resolve("")
            }
          } else{
            resolve("")
          }
      } else{
          resolve("")
      }  
  }

  @objc(getText:withRejecter:)
  func getText(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      //only available for iOS >= 13
      if #available(iOS 13, *){
          resolve(DocumentDataReader.getText())
      } else{
          resolve("")
      }
  }

  @objc(analyze:withRejecter:)
  func analyze(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void{
      //only available for iOS >= 13
      if #available(iOS 13, *){
          resolve(DocumentDataReader.analyze())
      } else{
          resolve("")
      }
  }

  @objc(setLogLevel:withResolver:withRejecter:)
  func setLogLevel(logLevel: Int, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock)-> Void{
      if let logLevel = Logger.Level(rawValue: UInt32(logLevel)){
        Logger.setLogLevel(level: logLevel)
      resolve(true)
    } else{
      resolve(false)
    }
  }

  @objc(getTraceFile:withRejecter:)
  func getTraceFile(resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock)-> Void{
    resolve(Logger.getTraceFile())
  }

  @objc(deleteTempFiles:withResolver:withRejecter:)
  func deleteTempFiles(deleteTraceFileContent:Bool,resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock)-> Void{
    resolve(DocutainSDK.deleteTempFiles(deleteTraceFileContent:deleteTraceFileContent))
  }

  internal func didFinishScan(withResult result: Bool) {
    if let resolve = scanResolve{
      resolve(result)
    } 
    //reset
    scanResolve = nil
  }

  private func mapAnalyzeConfiguration(config: [String: Any], analyzeConfig: AnalyzeConfiguration) -> Void{
    //print(config)

    if let readBIC = config["readBIC"] as? Bool{
      analyzeConfig.readBIC = readBIC
    }
    if let readPaymentState = config["readPaymentState"] as? Bool{
      analyzeConfig.readPaymentState = readPaymentState
    }
    //print("analyzeConfig.readBIC is \(analyzeConfig.readBIC)")
    //print("analyzeConfig.readPaymentState is \(analyzeConfig.readPaymentState)")
  }

  private func mapDocumentScannerConfiguration(config: [String: Any], scanConfig: DocumentScannerConfiguration) -> Void{
    //print(config)

    if let allowCaptureModeSetting = config["allowCaptureModeSetting"] as? Bool{
      scanConfig.allowCaptureModeSetting = allowCaptureModeSetting
    }
    if let pageEditConfig = config["pageEditConfig"] as? [String: Any]{
      if let allowPageFilter = pageEditConfig["allowPageFilter"] as? Bool {
        scanConfig.pageEditConfig.allowPageFilter = allowPageFilter
      }
      if let allowPageRotation = pageEditConfig["allowPageRotation"] as? Bool {
        scanConfig.pageEditConfig.allowPageRotation = allowPageRotation
      }
    }
    //print("scanConfig.allowCaptureModeSetting is \(scanConfig.allowCaptureModeSetting)")
    //print("scanConfig.allowPageFilter is \(scanConfig.pageEditConfig.allowPageFilter)")
    //print("scanConfig.allowPageRotation is \(scanConfig.pageEditConfig.allowPageRotation)")
  }
}
