import { NativeModules, Platform } from 'react-native';

import {PageEditConfiguration, DocumentScannerConfiguration, AnalyzeConfiguration, PDFPageFormat, LogLevel} from './model';

export {PageEditConfiguration, DocumentScannerConfiguration, AnalyzeConfiguration, PDFPageFormat, LogLevel};

const LINKING_ERROR =
  `The package 'react-native-docutain-sdk' doesn't seem to be linked. Make sure: \n\n` +
  Platform.select({ ios: "- You have run 'pod install'\n", default: '' }) +
  '- You rebuilt the app after installing the package\n' +
  '- You are not using Expo Go\n';

const DocutainSdk = NativeModules.DocutainSdk
  ? NativeModules.DocutainSdk
  : new Proxy(
      {},
      {
        get() {
          throw new Error(LINKING_ERROR);
        },
      }
    );

    
export function initSDK(licenseKey: string): Promise<boolean> {
  return DocutainSdk.initSDK(licenseKey);
}

export function scanDocument(options: DocumentScannerConfiguration): Promise<boolean> {
  return DocutainSdk.scanDocument(options);
}

export function loadFile(filePath: string): Promise<boolean> {
  return DocutainSdk.loadFile(filePath);
}

export function getText(): Promise<string> {
  return DocutainSdk.getText();
}

export function setAnalyzeConfiguration(config: AnalyzeConfiguration): Promise<boolean> {
  return DocutainSdk.setAnalyzeConfiguration(config);
}

export function analyze(): Promise<string> {
  return DocutainSdk.analyze();
}

export function writePDF(filePath: string, overWrite: boolean, PDF_PageFormat: PDFPageFormat): Promise<string> {
  const PageFormat: number  = PDF_PageFormat;
  return DocutainSdk.writePDF(filePath, overWrite, PageFormat);
}

export function getLastError(): Promise<string> {
  return DocutainSdk.getLastError();
}

export function setLogLevel(logLevel: LogLevel): Promise<boolean> {
  const Level: number  = logLevel;
  return DocutainSdk.setLogLevel(Level);
}

export function getTraceFile(): Promise<string> {
  return DocutainSdk.getTraceFile();
}

export function deleteTempFiles(deleteTraceFileContent: boolean): Promise<boolean> {
  return DocutainSdk.deleteTempFiles(deleteTraceFileContent);
}

export default DocutainSdk;
